<?php
final class OrdersAdmin extends Orders
{

  /**
  * Generate cache variables
  * @return void
  */
  public function generateCache( ){

    if( !is_file( DB_ORDERS ) )
      return null;

    $this->aOrdersFields = $GLOBALS['aOrdersFields'];
    $this->aOrdersExtFields = $GLOBALS['aOrdersExtFields'];
    $this->aOrdersProductsFields = $GLOBALS['aOrdersProductsFields'];

    $rFile = fopen( DB_ORDERS, 'r' );
    $i = 0;
    while( !feof( $rFile ) ){
      $sContent = fgets( $rFile );
      if( $i > 0 && !empty( $sContent ) ){
        $aData = unserialize( trim( $sContent ) );
        $this->aOrders[$aData['iOrder']] = $aData;
        $this->aOrders[$aData['iOrder']]['sDate'] = displayDate( $aData['iTime'] );
        $this->aOrders[$aData['iOrder']]['sStatus'] = $this->throwStatus( $aData['iStatus'] );        
      }
      $i++;
    } // end while
    fclose( $rFile );
  } // end function generateCache

  /**
  * Return orders list
  * @return string
  * @param string $sFile
  * @param int    $iList
  */
  public function listOrdersAdmin( $sFile, $iList = null ){
    if( !isset( $this->aOrders ) )
      $this->generateCache( );
    $oTpl = TplParser::getInstance( );
    $content= null;

    if( ( isset( $GLOBALS['sPhrase'] ) && !empty( $GLOBALS['sPhrase'] ) ) || ( isset( $_GET['iStatus'] ) && is_numeric( $_GET['iStatus'] ) ) ){
      $aOrders = $this->generateOrdersSearchListArray( $GLOBALS['sPhrase'], $GLOBALS['iStatus'] );
    }
    else{
      if( isset( $this->aOrders ) ){
        foreach( $this->aOrders as $iOrder => $aData ){
          $aOrders[] = $iOrder;
        } // end foreach
      }
    }

    if( isset( $aOrders ) ){
      rsort( $aOrders );
      $iCount = count( $aOrders );
      if( !isset( $iList ) ){
        $iList = $GLOBALS['config']['admin_list'];
      }

      $aKeys = countPageNumber( $iCount, ( isset( $_GET['iPage'] ) ? $_GET['iPage'] : null ), $iList );
      $this->mData = null;

      for( $i = $aKeys['iStart']; $i < $aKeys['iEnd']; $i++ ){
        $aData = $this->aOrders[$aOrders[$i]];

        if( $aData['iStatus'] == 1 )
          $aData['sStatus'] = '<b>'.$aData['sStatus'].'</b>';

        $aData['iStyle'] = ( $i % 2 ) ? 0: 1;
        $aData['sStyle'] = ( $i == ( $iCount - 1 ) ) ? 'L': $i + 1;
        $oTpl->setVariables( 'aData', $aData );
        $content .= $oTpl->tSubBlock( $sFile, 'ORDERS', 'LIST' );
      } // end for

      if( isset( $content ) ){
        $aData['sPages'] = countPagesClassic( $iCount, $iList, $aKeys['iPageNumber'], changeUri( $_SERVER['REQUEST_URI'] ) );
        $oTpl->setVariables( 'aData', $aData );
        return $oTpl->tSubBlock( $sFile, 'ORDERS', 'LIST', 'head' ).$content.$oTpl->tSubBlock( $sFile, 'ORDERS', 'LIST', 'foot' );
      }
    }
  } // end function listOrdersAdmin
  
  /**
  * Save orders status
  * @return void
  * @param array  $aForm
  */
  public function saveOrders( $aForm ){
    if( isset( $aForm['aStatus'] ) && is_array( $aForm['aStatus'] ) && $aForm['iStatus'] > 0 ){
      if( !isset( $this->aOrders ) )
        $this->generateCache( );

      foreach( $aForm['aStatus'] as $iOrder => $sValue ){
        if( $this->aOrders[$iOrder]['iStatus'] != $aForm['iStatus'] ){
          $aChange[$iOrder] = $aForm['iStatus'];
        }
      }
    }

    if( isset( $aChange ) ){
      $iTime = time( );

      $rFile = fopen( DB_ORDERS.'-backup', 'w' );
      fwrite( $rFile, '<?php exit; ?>'."\n" );
      foreach( $this->aOrders as $iOrder => $aValue ){
        if( isset( $aChange[$iOrder] ) )
          $aValue['iStatus'] = $aChange[$iOrder];
        fwrite( $rFile, serialize( compareArrays( $this->aOrdersFields, $aValue ) )."\n" );
      } // end foreach
      fclose( $rFile );

      $rFile1 = fopen( DB_ORDERS_EXT, 'r' );
      $rFile2 = fopen( DB_ORDERS_EXT.'-backup', 'w' );
      fwrite( $rFile2, '<?php exit; ?>'."\n" );
      $i = 0;
      while( !feof( $rFile1 ) ){
        $sContent = trim( fgets( $rFile1 ) );
        if( $i > 0 && !empty( $sContent ) ){
          $aData = unserialize( $sContent );
          if( isset( $aChange[$aData['iOrder']] ) ){
            $aData['aStatuses'][] = Array( 0 => $iTime, 1 => $aChange[$aData['iOrder']] );
            fwrite( $rFile2, serialize( $aData )."\n" );
          }
          else
            fwrite( $rFile2, $sContent."\n" );
        }
        $i++;
      } // end while
      fclose( $rFile1 );
      fclose( $rFile2 );

      $this->moveDatabaseFiles( );
      $this->generateCache( );
    }
  } // end function saveOrders

  /**
  * Return array with orders
  * @return array
  * @param string $sPhrase
  * @param int    $iStatus
  */
  private function generateOrdersSearchListArray( $sPhrase, $iStatus ){
    if( isset( $this->aOrders ) ){
      if( isset( $iStatus ) && !is_numeric( $iStatus ) )
        $iStatus = null;

      if( !empty( $sPhrase ) ){
        $aExp = explode( ' ', $sPhrase );
        $iCount = count( $aExp );
        for( $i = 0; $i < $iCount; $i++ ){
          $aExp[$i] = trim( $aExp[$i] );
          if( !empty( $aExp[$i] ) )
            $aWords[] = $aExp[$i];
        } // end for

        $iCount = count( $aWords );
      }

      foreach( $this->aOrders as $iOrder => $aData ){
        $bFound = true;

        if( isset( $iStatus ) && $iStatus != $aData['iStatus'] ){
          $bFound = null;
        }

        if( isset( $bFound ) && isset( $aWords ) ){
          $iFound = 0;

          $sSearchData = implode( ' ', $aData );

          for( $i = 0; $i < $iCount; $i++ ){
            //if( stristr( $sSearchData, $aWords[$i] ) )
            if( preg_match( '/'.preg_quote( $aWords[$i] ).'/ui', $sSearchData ) )
              $iFound++;
          } // end for

          if( $iFound != $iCount ){
            $bFound = null;
            $aNotFound[$iOrder] = true;
          }
        }

        if( isset( $bFound ) ){
          $aFound[$iOrder] = true;
        }
      }

      if( isset( $aNotFound ) && isset( $aWords ) && isset( $_GET['iProducts'] ) && $_GET['iProducts'] == 1 ){
        $rFile = fopen( DB_ORDERS_PRODUCTS, 'r' );
        $i2 = 0;
        while( !feof( $rFile ) ){
          $sContent = trim( fgets( $rFile ) );
          if( $i2 > 0 && !empty( $sContent ) ){
            $aData = unserialize( $sContent );
            if( isset( $aNotFound[$aData['iOrder']] ) ){
              $iFound = 0;
              for( $i = 0; $i < $iCount; $i++ ){
                if( stristr( $aData['sName'].' '.$aData['fPrice'], $aWords[$i] ) )
                  $iFound++;
              } // end for
              if( $iFound == $iCount )
                $aFound[$aData['iOrder']] = true;
            }
          }
          $i2++;
        } // end while
        fclose( $rFile );
      }

      if( isset( $aFound ) ){
        foreach( $this->aOrders as $iOrder => $aData ){
          if( isset( $aFound[$iOrder] ) )
            $aReturn[] = $iOrder;
        } // end foreach   
        return $aReturn;
      }
    }  
  } // end function generateOrdersSearchListArray

  /**
  * Delete order
  * @return void
  * @param int  $iOrder
  */
  public function deleteOrder( $iOrder ){
    if( isset( $this->aOrders[$iOrder] ) ){
      unset( $this->aOrders[$iOrder] );

      $rFile = fopen( DB_ORDERS.'-backup', 'w' );
      fwrite( $rFile, '<?php exit; ?>'."\n" );
      foreach( $this->aOrders as $iKey => $aValue ){
        fwrite( $rFile, serialize( compareArrays( $this->aOrdersFields, $aValue ) )."\n" );
      } // end foreach
      fclose( $rFile );

      $rFile1 = fopen( DB_ORDERS_EXT, 'r' );
      $rFile2 = fopen( DB_ORDERS_EXT.'-backup', 'w' );
      fwrite( $rFile2, '<?php exit; ?>'."\n" );
      $i = 0;
      while( !feof( $rFile1 ) ){
        $sContent = fgets( $rFile1 );
        if( $i > 0 && !empty( $sContent ) ){
          $aData = unserialize( trim( $sContent ) );
          if( $aData['iOrder'] != $iOrder ){
            fwrite( $rFile2, trim( $sContent )."\n" );
          }
        }
        $i++;
      } // end while
      fclose( $rFile1 );
      fclose( $rFile2 );

      $rFile1 = fopen( DB_ORDERS_PRODUCTS, 'r' );
      $rFile2 = fopen( DB_ORDERS_PRODUCTS.'-backup', 'w' );
      fwrite( $rFile2, '<?php exit; ?>'."\n" );
      $i = 0;
      while( !feof( $rFile1 ) ){
        $sContent = fgets( $rFile1 );
        if( $i > 0 && !empty( $sContent ) ){
          $aData = unserialize( trim( $sContent ) );
          if( $aData['iOrder'] != $iOrder ){
            fwrite( $rFile2, trim( $sContent )."\n" );
          }
        }
        $i++;
      } // end while
      fclose( $rFile1 );
      fclose( $rFile2 );

      $this->moveDatabaseFiles( );

    }
  } // end function deleteOrder

  /**
  * Save order data
  * @return void
  * @param array  $aForm
  */
  public function saveOrder( $aForm ){
    $aOrderBefore = $this->aOrders[$aForm['iOrder']];
    $this->aOrders[$aForm['iOrder']] = $aForm = array_merge( $this->aOrders[$aForm['iOrder']], changeMassTxt( $aForm, 'H', Array( 'sComment', 'LenHNds' ) ) );

    $rFile = fopen( DB_ORDERS.'-backup', 'w' );
    fwrite( $rFile, '<?php exit; ?>'."\n" );
    foreach( $this->aOrders as $iKey => $aValue ){
      fwrite( $rFile, serialize( compareArrays( $this->aOrdersFields, $aValue ) )."\n" );
    } // end foreach
    fclose( $rFile );

    $rFile1 = fopen( DB_ORDERS_EXT, 'r' );
    $rFile2 = fopen( DB_ORDERS_EXT.'-backup', 'w' );
    fwrite( $rFile2, '<?php exit; ?>'."\n" );
    $i = 0;
    while( !feof( $rFile1 ) ){
      $sContent = trim( fgets( $rFile1 ) );
      if( $i > 0 && !empty( $sContent ) ){
        $aData = unserialize( $sContent );
        if( $aData['iOrder'] == $aForm['iOrder'] ){
          if( $aForm['iStatus'] != $aOrderBefore['iStatus'] ){
            $aData['aStatuses'][] = Array( 0 => time( ), 1 => $aForm['iStatus'] );
          }
          
          $aStatuses = isset( $aData['aStatuses'] ) ? Array( 'aStatuses' => $aData['aStatuses'] ) : null;
          if( isset( $aStatuses ) )
            fwrite( $rFile2, serialize( array_merge( compareArrays( $this->aOrdersExtFields, $aForm ), $aStatuses ) )."\n" );  
          else
            fwrite( $rFile2, serialize( compareArrays( $this->aOrdersExtFields, $aForm ) )."\n" );  
          
        }
        else{
          fwrite( $rFile2, $sContent."\n" );
        }
      }
      $i++;
    } // end while
    fclose( $rFile1 );
    fclose( $rFile2 );
    unset( $this->aOrders );

    $this->generateProducts( $aForm['iOrder'] );
    if( isset( $this->aProducts ) ){
      if( isset( $aForm['aProductsDelete'] ) ){
        $bChangeProducts = true;
      }
      else{
        foreach( $this->aProducts as $iElement => $aData ){
          if( isset( $aForm['aProducts'][$iElement] ) && ( $aForm['aProducts'][$iElement]['sName'] != $aData['sName'] || $aForm['aProducts'][$iElement]['fPrice'] != $aData['fPrice'] || $aForm['aProducts'][$iElement]['iQuantity'] != $aData['iQuantity'] ) ){
            $bChangeProducts = true;
            break;
          }
        } // end foreach
      }

      if( !empty( $aForm['aNewProduct']['iProduct'] ) && !empty( $aForm['aNewProduct']['sName'] ) && !empty( $aForm['aNewProduct']['fPrice'] ) && !empty( $aForm['aNewProduct']['iQuantity'] ) ){
        $bChangeProducts = true;
        $bAddProduct = true;
      }

      if( isset( $bChangeProducts ) ){
        $rFile1 = fopen( DB_ORDERS_PRODUCTS, 'r' );
        $rFile2 = fopen( DB_ORDERS_PRODUCTS.'-backup', 'w' );
        fwrite( $rFile2, '<?php exit; ?>'."\n" );
        $i = 0;
        $iLastElement = 0;
        while( !feof( $rFile1 ) ){
          $sContent = trim( fgets( $rFile1 ) );
          if( $i > 0 && !empty( $sContent ) ){
            $aData = unserialize( $sContent );
            if( $aData['iOrder'] == $aForm['iOrder'] ){
              if( !isset( $aForm['aProductsDelete'][$aData['iElement']] ) ){
                fwrite( $rFile2, serialize( compareArrays( $this->aOrdersProductsFields, array_merge( $aData, changeMassTxt( $aForm['aProducts'][$aData['iElement']] ) ) ) )."\n" );
              }
            }
            else{
              fwrite( $rFile2, $sContent."\n" );
            }
            if( $aData['iElement'] > $iLastElement )
              $iLastElement = $aData['iElement'];
          }
          $i++;
        } // end while

        if( isset( $bAddProduct ) ){
          $aAdd = $aForm['aNewProduct'];
          $aAdd['iElement'] = ( $iLastElement + 1 );
          $aAdd['fPrice'] = normalizePrice( $aAdd['fPrice'] );
          $aAdd['sName'] = trim( $aAdd['sName'] );
          $aAdd['iOrder'] = $aForm['iOrder'];
          fwrite( $rFile2, serialize( compareArrays( $this->aOrdersProductsFields, changeMassTxt( $aAdd, 'H' ) ) )."\n" );
        }
        fclose( $rFile1 );
        fclose( $rFile2 );
      }
    }
    $this->moveDatabaseFiles( );

  } // end function saveOrder

  /**
  * List all payment and shipping methods
  * @return string
  * @param string $sFile
  * @param int $iType
  */
  public function listPaymentsShippingAdmin( $sFile, $iType = 1 ){
    $oTpl = TplParser::getInstance( );
    $content = null;
    $aData = $this->throwPaymentsShipping( $iType );
    $sBlock = ( $iType == 1 ) ? 'PAYMENTS' : 'SHIPPING';
    
    if( isset( $aData ) ){
      if( $iType == 2 )
        $aPayments = $this->throwPaymentsShipping( 1 );

      $i = 0;
      foreach( $aData as $iKey => $aPaymentShipping ){
        $aPaymentShipping['iStyle'] = ( $i % 2 ) ? 0: 1;
        $aPaymentShipping['sStatus'] = throwYesNoTxt( isset( $aPaymentShipping['iStatus'] ) ? $aPaymentShipping['iStatus'] : 0 );
        $aPaymentShipping['sPayments'] = null;

        if( isset( $aPayments ) && isset( $aPaymentShipping['aPayments'] ) ){
          foreach( $aPaymentShipping['aPayments'] as $iPayment => $mPrice ){
            if( isset( $aPayments[$iPayment] ) )
              $aPaymentShipping['sPayments'] .= $aPayments[$iPayment]['sName'].', ';
          } // end foreach
        }

        $oTpl->setVariables( 'aData', $aPaymentShipping );
        $content .= $oTpl->tSubBlock( $sFile, $sBlock, 'LIST' );
        $i++;
      } // end foreach
      return $oTpl->tSubBlock( $sFile, $sBlock, 'LIST', 'head' ).$content.$oTpl->tSubBlock( $sFile, $sBlock, 'LIST', 'foot' );
    }
  } // end function listPaymentsShippingAdmin

  /**
  * Delete payment and shipping data
  * @return void
  * @param int $iId
  */
  public function deletePaymentShipping( $iId ){
    $oFFS = FlatFilesSerialize::getInstance( );
    $aPaymentsShipping = $oFFS->getData( DB_PAYMENTS_SHIPPING, true );
    unset( $aPaymentsShipping[$iId] );
    $oFFS->saveData( DB_PAYMENTS_SHIPPING, $aPaymentsShipping );
  } // end function deletePaymentShipping

  /**
  * Save payment or shipping data
  * @return void
  * @param array $aForm
  * @param int $iType
  */
  public function savePaymentShipping( $aForm, $iType = 1 ){
    $oFFS = FlatFilesSerialize::getInstance( );

    $aPaymentsShipping = $oFFS->getData( DB_PAYMENTS_SHIPPING, true );

    if( isset( $aForm['iId'] ) && is_numeric( $aForm['iId'] ) ){
    }
    else{
      $aForm['iId'] = $oFFS->throwLastId( DB_PAYMENTS_SHIPPING, 'iId' ) + 1;
    }    

    if( !isset( $aForm['iStatus'] ) )
      $aForm['iStatus'] = 0;

    if( isset( $aForm['sPrice'] ) && is_numeric( str_replace( ',', '.', $aForm['sPrice'] ) ) )
      $aForm['sPrice'] = normalizePrice( $aForm['sPrice'] );

    $aPaymentsShipping[$aForm['iId']] = compareArrays( $GLOBALS['aPaymentsShippingFields'], changeMassTxt( $aForm, '' ) );
    $aPaymentsShipping[$aForm['iId']]['iType'] = $iType;
    if( isset( $aPaymentsShipping[$aForm['iId']]['aPayments'] ) )
      unset( $aPaymentsShipping[$aForm['iId']]['aPayments'] );

    if( isset( $aForm['aPayments'] ) ){
      foreach( $aForm['aPayments'] as $iId => $iValue ){
        $mPrice = isset( $aForm['aPaymentsPrices'][$iId] ) ? $aForm['aPaymentsPrices'][$iId] : null;
        $aPaymentsShipping[$aForm['iId']]['aPayments'][$iId] = $mPrice;
      }
    }

    $oFFS->saveData( DB_PAYMENTS_SHIPPING, $aPaymentsShipping );

    return $aForm['iId'];
  } // end function savePaymentShipping

  /**
  * Return list of shipping payments
  * @return string
  * @param string $sFile
  * @param int $iId
  */
  public function listShippingPaymentsForm( $sFile, $iId = null ){
    $oTpl = TplParser::getInstance( );
    $oFFS = FlatFilesSerialize::getInstance( );
    $content = null;

    $aPayments = $this->throwPaymentsShipping( 1 );
    if( isset( $aPayments ) ){
      $aShipping = $this->throwPaymentsShipping( 2 );
      if( isset( $iId ) && isset( $aShipping[$iId] ) && isset( $aShipping[$iId]['aPayments'] ) ){
        $aCheck = $aShipping[$iId]['aPayments'];
      }

      $i = 0;
      foreach( $aPayments as $iKey => $aData ){
        if( isset( $aCheck[$iKey] ) ){
          $oTpl->setIf( 'CHECKED' );
          $aData['mPrice'] = $aCheck[$iKey];
        }
        else{
          $oTpl->setIf( 'DISABLE' );
        }

        $aData['iStyle'] = ( $i % 2 ) ? 0: 1;
        $aData['sStatus'] = throwYesNoTxt( isset( $aData['iStatus'] ) ? $aData['iStatus'] : 0 );
        $oTpl->setVariables( 'aData', $aData );
        $content .= $oTpl->tSubBlock( $sFile, 'SHIPPING_PAYMENTS', 'LIST' );
        $i++;
      } // end foreach

      return $oTpl->tSubBlock( $sFile, 'SHIPPING_PAYMENTS', 'LIST', 'head' ).$content.$oTpl->tSubBlock( $sFile, 'SHIPPING_PAYMENTS', 'LIST', 'foot' );
    }
  } // end function listShippingPaymentsForm

  /**
  * Return status list
  * @return string
  * @param string $sFile
  * @param int    $iOrder
  */
  public function listOrderStatuses( $sFile, $iOrder ){
    if( isset( $this->aOrders[$iOrder]['aStatuses'] ) ){
      $oTpl = TplParser::getInstance( );
      $content = null;
      $aStatuses = $this->aOrders[$iOrder]['aStatuses'];
      rsort( $aStatuses );

      foreach( $aStatuses as $iKey => $aValue ){
        $oTpl->setVariables( 'aData', Array( 'sDate' => displayDate( $aValue[0] ), 'sStatus' => $this->throwStatus( $aValue[1] ) ) );
        $content .= $oTpl->tSubBlock( $sFile, 'STATUSES', 'LIST' );      
      } // end foreach

      if( isset( $content ) ){
        return $oTpl->tSubBlock( $sFile, 'STATUSES', 'LIST', 'head' ).$content.$oTpl->tSubBlock( $sFile, 'STATUSES', 'LIST', 'foot' );
      }
    }
  } // end function listOrderStatuses

  /**
  * Moves database files from backup to normal file
  * @return void
  */
  private function moveDatabaseFiles( ){

    if( is_file( DB_ORDERS_PRODUCTS.'-backup' ) ){
      unlink( DB_ORDERS_PRODUCTS );
      rename( DB_ORDERS_PRODUCTS.'-backup', DB_ORDERS_PRODUCTS );
      chmod( DB_ORDERS_PRODUCTS, FILES_CHMOD );
    }

    if( is_file( DB_ORDERS_EXT.'-backup' ) ){
      unlink( DB_ORDERS_EXT );
      rename( DB_ORDERS_EXT.'-backup', DB_ORDERS_EXT );
      chmod( DB_ORDERS_EXT, FILES_CHMOD );
    }

    if( is_file( DB_ORDERS.'-backup' ) ){
      unlink( DB_ORDERS );
      rename( DB_ORDERS.'-backup', DB_ORDERS );
      chmod( DB_ORDERS, FILES_CHMOD );
    }
  } // end function moveDatabaseFiles

  /**
  * List last added orders
  * @return string
  * @param string $sFile
  */
  public function listLastOrders( $sFile ){
    if( !isset( $this->aOrders ) )
      $this->generateCache( );
    $oTpl = TplParser::getInstance( );
    $content= null;

    if( isset( $this->aOrders ) ){
      foreach( $this->aOrders as $iOrder => $aData ){
        $aOrders[] = $iOrder;
      } // end foreach
    }

    if( isset( $aOrders ) ){
      rsort( $aOrders );
      $iMax = 5;
      $iCount = count( $aOrders );
      if( $iCount > $iMax )
        $iCount = $iMax;
      
      $content = null;

      for( $i = 0; $i < $iCount; $i++ ){
        $aData = $this->aOrders[$aOrders[$i]];
        $oTpl->setIf( 'DATA' );
        $oTpl->setVariables( 'aData', Array( 'sName' => $aData['sFirstName'].' '.$aData['sLastName'], 'sData' => $aData['sDate'], 'sLink' => '?p=orders-form&amp;iOrder='.$aData['iOrder'], 'iId' => $aData['iOrder'] ) );
        $content .= $oTpl->tSubBlock( $sFile, 'EVENTS', 'LIST' );
      } // end for
      
      $oTpl->setIf( 'ORDERS' );
      unset( $this->aOrders );
      return $oTpl->tSubBlock( $sFile, 'EVENTS', 'LIST', 'head' ).$content.$oTpl->tSubBlock( $sFile, 'EVENTS', 'LIST', 'foot' );
    }
  } // end function listLastOrders
};
?>