<?php
final class ProductsAdmin extends Products
{
  private static $oInstance = null;

  public static function getInstance( ){  
    if( !isset( self::$oInstance ) ){  
      self::$oInstance = new ProductsAdmin( );  
    }  
    return self::$oInstance;  
  } // end function getInstance

  /**
  * Constructor
  * @return void
  */
  private function __construct( ){
    $this->generateCache( );
  } // end function __construct
  
  /**
  * List products
  * @return string
  * @param string $sFile
  * @param int    $iContent
  */
  public function listProductsAdmin( $sFile, $iList = null ){
    $oTpl = TplParser::getInstance( );
    $oFile = Files::getInstance( );
    $content= null;

    if( isset( $GLOBALS['iPageSearch'] ) && is_numeric( $GLOBALS['iPageSearch'] ) ){
      $oPage = Pages::getInstance( );
      $aData = $oPage->throwAllChildrens( $GLOBALS['iPageSearch'] );
      if( isset( $aData ) ){
        foreach( $aData as $iValue ){
          $this->aPages[$iValue] = $iValue;
        }
      }
      $this->aPages[$GLOBALS['iPageSearch']] = $GLOBALS['iPageSearch'];
    }

    if( isset( $GLOBALS['sPhrase'] ) && !empty( $GLOBALS['sPhrase'] ) ){
      $aProducts = $this->generateProductsSearchListArray( $GLOBALS['sPhrase'] );
    }
    elseif( isset( $this->aPages ) ){
      $aProducts = $this->generateProductsListArray( );
    }
    else{
      if( isset( $this->aProducts ) ){
        foreach( $this->aProducts as $iProduct => $aData ){
          $aProducts[] = $iProduct;
        } // end foreach
      }
    }

    if( isset( $aProducts ) ){
      if( isset( $_GET['sSort'] ) && !empty( $_GET['sSort'] ) && $_GET['sSort'] != 'position' ){
        $aProducts = $this->sortProducts( $aProducts, $_GET['sSort'] );
      }

      $iCount = count( $aProducts );
      if( !isset( $iList ) ){
        $iList = $GLOBALS['config']['admin_list'];
      }

      $aKeys = countPageNumber( $iCount, ( isset( $_GET['iPage'] ) ? $_GET['iPage'] : null ), $iList );
      $this->mData = null;

      for( $i = $aKeys['iStart']; $i < $aKeys['iEnd']; $i++ ){
        $aData = $this->aProducts[$aProducts[$i]];

        $aData['iStyle'] = ( $i % 2 ) ? 0: 1;
        $aData['sPrice'] = displayPrice( $aData['mPrice'] );
        $aData['sPages'] = str_replace( '|', '&nbsp;|&nbsp;', strip_tags( $this->throwProductsPagesTree( $aData['iProduct'] ) ) );
        $aData['sDescriptionShort'] = changeTxt( $aData['sDescriptionShort'], 'nlNds' );
        if( $aData['iStatus'] == 1 )
          $oTpl->setIf( 'CHECKED' );
        $oTpl->setVariables( 'aData', $aData );
        $content .= $oTpl->tSubBlock( $sFile, 'PRODUCTS', 'LIST' );
      } // end for

      if( isset( $content ) ){
        $aData['sPages'] = countPagesClassic( $iCount, $iList, $aKeys['iPageNumber'], changeUri( $_SERVER['REQUEST_URI'] ) );
        $oTpl->setVariables( 'aData', $aData );
        return $oTpl->tSubBlock( $sFile, 'PRODUCTS', 'LIST', 'head' ).$content.$oTpl->tSubBlock( $sFile, 'PRODUCTS', 'LIST', 'foot' );
      }
    }
  } // end function listProductsAdmin

  /**
  * Save products position, status and price
  * @return void
  * @param array  $aForm
  */
  public function saveProducts( $aForm ){
    if( isset( $aForm['aPositions'] ) && is_array( $aForm['aPositions'] ) ){
      foreach( $this->aProducts as $iProduct => $aData ){
        if( isset( $aForm['aPositions'][$iProduct] ) ){
          $aForm['aPositions'][$iProduct] = trim( $aForm['aPositions'][$iProduct] );
          $iStatus = isset( $aForm['aStatus'][$iProduct] ) ? 1 : 0;
          
          if( is_numeric( str_replace( ',', '.', $aForm['aPrices'][$iProduct] ) ) )
            $aForm['aPrices'][$iProduct] = normalizePrice( trim( $aForm['aPrices'][$iProduct] ) );
          else
            $aForm['aPrices'][$iProduct] = trim( $aForm['aPrices'][$iProduct] );

          if( is_numeric( $aForm['aPositions'][$iProduct] ) && $aForm['aPositions'][$iProduct] != $aData['iPosition'] ){
            $this->aProducts[$iProduct]['iPosition'] = $aForm['aPositions'][$iProduct];
            $bChanged = true;
          }
          if( $iStatus != $aData['iStatus'] ){
            $this->aProducts[$iProduct]['iStatus'] = $iStatus;
            $bChanged = true;
          }
          if( $aForm['aPrices'][$iProduct] != $aData['mPrice'] ){
            $this->aProducts[$iProduct]['mPrice'] = $aForm['aPrices'][$iProduct];
            $bChanged = true;
          }
        }
      } // end foreach

      if( isset( $bChanged ) ){
        $oFFS = FlatFilesSerialize::getInstance( );
        $oFFS->saveData( DB_PRODUCTS, $this->createArray( ) );
        $this->generateCache( );
      }
    }
  } // end function saveProducts

  /**
  * Save product data
  * @return int
  * @param array  $aForm
  */
  public function saveProduct( $aForm ){
    $oFile = FilesAdmin::getInstance( );
    $oFFS = FlatFilesSerialize::getInstance( );

    if( isset( $aForm['iProduct'] ) && is_numeric( $aForm['iProduct'] ) && isset( $this->aProducts[$aForm['iProduct']] ) ){
    }
    else{
      $aForm['iProduct'] = $oFFS->throwLastId( DB_PRODUCTS, 'iProduct' ) + 1;
    }


    if( empty( $aForm['sTemplate'] ) || $aForm['sTemplate'] == $GLOBALS['config']['default_products_template'] )
      unset( $aForm['sTemplate'] );

    if( empty( $aForm['sTheme'] ) || $aForm['sTheme'] == $GLOBALS['config']['default_theme'] )
      unset( $aForm['sTheme'] );

    if( !isset( $aForm['iPosition'] ) || !is_numeric( $aForm['iPosition'] ) || $aForm['iPosition'] < -99 || $aForm['iPosition'] > 999 )
      $aForm['iPosition'] = 0;

    if( !isset( $aForm['iStatus'] ) )
      $aForm['iStatus'] = 0;

    if( is_numeric( str_replace( ',', '.', $aForm['mPrice'] ) ) )
      $aForm['mPrice'] = normalizePrice( $aForm['mPrice'] );

    $aForm = changeMassTxt( $aForm, '', Array( 'sDescriptionShort', 'Nds' ), Array( 'sDescriptionFull', 'Nds' ), Array( 'sMetaDescription', 'Nds' ) );

    if( isset( $aForm['aPages'] ) && is_array( $aForm['aPages'] ) ){
      if( isset( $this->aProductsPages[$aForm['iProduct']] ) )
        unset( $this->aProductsPages[$aForm['iProduct']] );
      foreach( $aForm['aPages'] as $iPage ){
        $this->aProductsPages[$aForm['iProduct']][$iPage] = (int) $iPage;
      } // end foreach
      $oFFS->saveData( DB_PRODUCTS_PAGES, $this->aProductsPages );
    }

    // deleting keys from array $aForm that not exists in $aProductsFields in database/_fields.php
    $this->aProducts[$aForm['iProduct']] = $aForm;

    // START OF PLUGIN LICENSE RESTRICTIONS - GET MORE INFO HERE: read-me.txt
    if( isset( $aForm['iShortLink'] ) )
      saveShortLink( change2Url( !empty( $aForm['sNameUrl'] ) ? $aForm['sNameUrl'] : $aForm['sName'] ), $aForm['iProduct'], 2 );
    else
      deleteShortLink( $aForm['iProduct'], 2 );
    // END OF PLUGIN LICENSE RESTRICTIONS

    if( isset( $aForm['aFilesDescription'] ) )
      $oFile->saveFiles( $aForm, 2, $aForm['iProduct'] );
    if( isset( $_FILES['aNewFiles'] ) )
      $oFile->addFilesUploaded( $aForm, $aForm['iProduct'], 2, 'iProduct' );
    if( isset( $aForm['aDirFiles'] ) )
      $oFile->addFilesFromServer( $aForm, $aForm['iProduct'], 2, 'iProduct' );  

    $oFFS->saveData( DB_PRODUCTS, $this->createArray( ) );

    $this->generateCache( );
    return $aForm['iProduct'];
  } // end function saveProduct

  /**
  * Delete product
  * @return void
  * @param int  $iProduct
  * @param bool $bWithoutFiles
  */
  public function deleteProduct( $iProduct, $bWithoutFiles ){

    if( isset( $this->aProducts[$iProduct] ) ){
      unset( $this->aProducts[$iProduct] );
      $oFile = FilesAdmin::getInstance( );
      $oFFS = FlatFilesSerialize::getInstance( );
      
      $aSave = $this->createArray( $this->aProducts );
      $oFFS->saveData( DB_PRODUCTS, $aSave );

      if( isset( $this->aProductsPages[$iProduct] ) ){
        unset( $this->aProductsPages[$iProduct] );
        $oFFS->saveData( DB_PRODUCTS_PAGES, $this->aProductsPages );
      }

      $oFile->deleteFiles( Array( $iProduct => $iProduct ), 2, 'iProduct', $bWithoutFiles );
      // START OF PLUGIN LICENSE RESTRICTIONS - GET MORE INFO HERE: read-me.txt
      deleteShortLink( $iProduct, 2 );
      // END OF PLUGIN LICENSE RESTRICTIONS
    }

  } // end function deleteProduct

  /**
  * List last added pages
  * @return string
  * @param string $sFile
  */
  public function listLastProducts( $sFile ){
    if( isset( $this->aProducts) ){
      $oTpl = TplParser::getInstance( );

      $iMax = 5;
      $aProducts = $this->sortProducts( array_keys( $this->aProducts ) );
      $iCount = count( $aProducts );
      if( $iCount > $iMax )
        $iCount = $iMax;
      
      $content = null;

      for( $i = 0; $i < $iCount; $i++ ){
        $aData = $this->aProducts[$aProducts[$i]];
        $aData['sData'] = $aData['mPrice'];
        $aData['sLink'] = '?p=products-form&amp;iProduct='.$aData['iProduct'];
        $aData['iId'] = $aData['iProduct'];
        $oTpl->setIf( 'DATA' );
        $oTpl->setVariables( 'aData', $aData );
        $content .= $oTpl->tSubBlock( $sFile, 'EVENTS', 'LIST' );
      } // end for
      
      $oTpl->setIf( 'PRODUCTS' );

      return $oTpl->tSubBlock( $sFile, 'EVENTS', 'LIST', 'head' ).$content.$oTpl->tSubBlock( $sFile, 'EVENTS', 'LIST', 'foot' );
    }
  } // end function listLastProducts

  /**
  * Function creates array to save to file
  * @return array
  */
  protected function createArray( ){
    if( isset( $this->aProducts ) ){
      // Sorting array before it will save to file 
      foreach( $this->aProducts as $iKey => $aValue ){
        $aSort[$iKey][0] = (int) $aValue['iPosition'];
        $aSort[$iKey][1] = $aValue['sName'];
        $aSort[$iKey][2] = $aValue['iProduct'];
      } // end foreach

      if( isset( $aSort ) ){
        sort( $aSort );
        
        foreach( $aSort as $iKey => $aValue ){
          $aSave[] = compareArrays( $this->aFields, $this->aProducts[$aValue[2]] );
        } // end foreach

        return $aSave;
      }
    }
  } // end function createArray

  /**
  * Returns products to csv
  * @return string
  * @param string $sSep
  */
  // START OF PLUGIN LICENSE RESTRICTIONS - GET MORE INFO HERE: read-me.txt
  public function exportProducts( $sSep ){
    global $lang;
    if( isset( $this->aProducts ) ){
      $sExport = strip_tags( $lang['Name'].$sSep.$lang['Price'].$sSep.$lang['Position'].$sSep.$lang['Status'].$sSep.$lang['Product_available'].$sSep.$lang['Page_title'].$sSep.$lang['Key_words'].$sSep.$lang['Pages'].$sSep.$lang['Id'] )."\n";

      foreach( $this->aProducts as $aData ){
        $aExport = Array( $aData['sName'], $aData['mPrice'], $aData['iPosition'], $aData['iStatus'], $aData['sAvailable'], $aData['sNameTitle'], $aData['sMetaKeywords'], implode( $this->aProductsPages[$aData['iProduct']], ',' ), $aData['iProduct'] );
        $sExport .= preg_replace( '/\|n\|/', '', implode( $sSep, $aExport ) )."\n";
      } // end foreach
      return $sExport;
    }
  } // end function exportProducts
  // END OF PLUGIN LICENSE RESTRICTIONS

};
?>