<?php
class Products
{

  public $aProducts = null;
  public $aProductsPages = null;
  public $aPages = null;
  protected $mData = null;
  protected $aFields = null;
  private static $oInstance = null;

  public static function getInstance( ){  
    if( !isset( self::$oInstance ) ){  
      self::$oInstance = new Products( );  
    }  
    return self::$oInstance;  
  } // end function getInstance

  /**
  * Constructor
  * @return void
  */
  private function __construct( ){
    $this->generateCache( );
  } // end function __construct

  /**
  * Generate cache variables
  * @return void
  */
  public function generateCache( ){

    if( !is_file( DB_PRODUCTS ) )
      return null;

    $this->aFields = $GLOBALS['aProductsFields'];

    $oPage = Pages::getInstance( );
    $oFFS = FlatFilesSerialize::getInstance( );
    
    $aPages = $oFFS->getData( DB_PRODUCTS_PAGES );
    $aData = $oFFS->getData( DB_PRODUCTS );

    if( !is_array( $aData ) || ( is_array( $aData ) && count( $aData ) == 0 ) )
      return null;

    $iStatus = throwStatus( );
    $sLanguageUrl = ( LANGUAGE_IN_URL == true ) ? LANGUAGE.LANGUAGE_SEPARATOR : null;
    // START OF PLUGIN LICENSE RESTRICTIONS - GET MORE INFO HERE: read-me.txt
    $aShortLinks = throwShortLinks( 2 );
    // END OF PLUGIN LICENSE RESTRICTIONS
    $this->aProducts = null;
    $this->aProductsPages = null;

    foreach( $aData as $iKey => $aValue ){
      if( isset( $aValue['iStatus'] ) && $aValue['iStatus'] >= $iStatus && ( !defined( 'CUSTOMER_PAGE' ) || isset( $aPages[$aValue['iProduct']] ) ) ){
        $this->aProducts[$aValue['iProduct']] = $aValue;
        // START OF PLUGIN LICENSE RESTRICTIONS - GET MORE INFO HERE: read-me.txt
        if( is_numeric( $this->aProducts[$aValue['iProduct']]['mPrice'] ) && !empty( $GLOBALS['config']['vat'] ) ){
          $fVat = '1.'.str_replace( '.', '', ( $GLOBALS['config']['vat'] < 10 ) ? '0'.$GLOBALS['config']['vat'] : $GLOBALS['config']['vat'] );
          $this->aProducts[$aValue['iProduct']]['fPriceNetto'] = normalizePrice( ( $this->aProducts[$aValue['iProduct']]['mPrice'] / $fVat ) );
          $this->aProducts[$aValue['iProduct']]['sPriceNetto'] = displayPrice( $this->aProducts[$aValue['iProduct']]['fPriceNetto'] );
        }
        // END OF PLUGIN LICENSE RESTRICTIONS
        $sUrlName = $sLanguageUrl.change2Url( !empty( $this->aProducts[$aValue['iProduct']]['sNameUrl'] ) ? $this->aProducts[$aValue['iProduct']]['sNameUrl'] : $this->aProducts[$aValue['iProduct']]['sName'] );
        if( is_numeric( $sUrlName ) )
          $sUrlName .= '-';
        $this->aProducts[$aValue['iProduct']]['sLinkName'] = '?'.$aValue['iProduct'].','.$sUrlName;
        $this->aProductsPages[$aValue['iProduct']] = isset( $aPages[$aValue['iProduct']] ) ? $aPages[$aValue['iProduct']] : null;

        // START OF PLUGIN LICENSE RESTRICTIONS - GET MORE INFO HERE: read-me.txt
        if( isset( $aShortLinks[$aValue['iProduct']] ) ){
          $this->aProducts[$aValue['iProduct']]['sLinkNameRaw'] = $this->aProducts[$aValue['iProduct']]['sLinkName'];
          $this->aProducts[$aValue['iProduct']]['sLinkName'] = '?'.$aShortLinks[$aValue['iProduct']];
          $this->aProducts[$aValue['iProduct']]['iShortLink'] = true;
        }
        // END OF PLUGIN LICENSE RESTRICTIONS
      }
    } // end foreach
  } // end function generateCache


  /**
  * List products
  * @return string
  * @param string $sFile
  * @param int    $iContent
  * @param int    $iList
  * @param array  $aProducts
  * @param string $sBlock
  */
  public function listProducts( $sFile, $iContent, $iList = null, $aProducts = null, $sBlock = 'PRODUCTS' ){
    $oTpl = TplParser::getInstance( );
    $oFile = Files::getInstance( );
    $oPage = Pages::getInstance( );
    $content= null;
    $sUrlExt= null;

    if( !isset( $aProducts ) ){
      if( isset( $GLOBALS['sPhrase'] ) && !empty( $GLOBALS['sPhrase'] ) ){
        $aProducts = $this->generateProductsSearchListArray( $GLOBALS['sPhrase'] );
        $sUrlExt .= '&amp;sPhrase='.$GLOBALS['sPhrase'];
      }
      else{
        if( DISPLAY_SUBCATEGORY_PRODUCTS === true ){
          // return all pages and subpages
          $aData = $oPage->throwAllChildrens( $iContent );
          if( isset( $aData ) ){
            foreach( $aData as $iValue ){
              $this->aPages[$iValue] = $iValue;
            }
          }
        }
        $this->aPages[$iContent] = $iContent;
        $aProducts = $this->generateProductsListArray( );
      }
    }

    if( isset( $aProducts ) ){

      $sSort = isset( $_GET['sSort'] ) ? $_GET['sSort'] : null;
      if( isset( $sSort ) && !empty( $sSort ) && in_array( $sSort, Array( 'price', 'name' ) ) ){
        $aProducts = $this->sortProducts( $aProducts, $sSort );
        $sUrlExt .= '&amp;sSort='.$sSort;
      }
      else
        $sSort = null;

      $sBasketPage = ( isset( $GLOBALS['config']['basket_page'] ) && isset( $oPage->aPages[$GLOBALS['config']['basket_page']] ) ) ? $oPage->aPages[$GLOBALS['config']['basket_page']]['sLinkName'] : null;

      $iCount = count( $aProducts );
      if( !isset( $iList ) ){
        $iList = $GLOBALS['config']['products_list'];
      }

      $aKeys = countPageNumber( $iCount, ( isset( $GLOBALS['aActions']['o2'] ) ? $GLOBALS['aActions']['o2'] : null ), $iList );
      $this->mData = null;

      for( $i = $aKeys['iStart']; $i < $aKeys['iEnd']; $i++ ){
        $aData = $this->aProducts[$aProducts[$i]];

        $aData['iStyle'] = ( $i % 2 ) ? 0: 1;
        $aData['iStyleColums'] = ( $i % 3 ) ? 0: 1;
        $aData['sStyle'] = ( $i == ( $aKeys['iEnd'] - 1 ) ) ? 'L': $i + 1;
        $aData['sPrice'] = is_numeric( $aData['mPrice'] ) ? displayPrice( $aData['mPrice'] ) : $aData['mPrice'];
        $aData['sPages'] = $this->throwProductsPagesTree( $aData['iProduct'] );

        if( !empty( $aData['sDescriptionShort'] ) ){
          $aData['sDescriptionShort'] = changeTxt( $aData['sDescriptionShort'], 'nlNds' );
          $oTpl->setIf( 'DESCRIPTION' );
        }

        if( isset( $oFile->aImagesDefault[2][$aData['iProduct']] ) ){
          $aDataImage = $oFile->aFilesImages[2][$oFile->aImagesDefault[2][$aData['iProduct']]];
          $oTpl->setVariables( 'aDataImage', $aDataImage );
          $oTpl->setIf( 'IMAGE' );
        }

        if( is_numeric( $aData['mPrice'] ) ){
          if( isset( $sBasketPage ) ){
            $aData['sBasketPage'] = $sBasketPage;
            $oTpl->setIf( 'BASKET' );
          }
          $oTpl->setVariables( 'aData', $aData );
          $oTpl->setIf( 'PRICE' );
        }
        else{
          $oTpl->setIf( 'NO-PRICE' );
        }

        $oTpl->setVariables( 'aData', $aData );
        $content .= $oTpl->tSubBlock( $sFile, $sBlock, 'LIST' );
      } // end for

      if( isset( $content ) ){
        if( $iCount > $iList ){
          $aData['sPages'] = countPages( $iCount, $iList, $aKeys['iPageNumber'], ( isset( $oPage->aPages[$iContent]['sLinkNameHome'] ) ? $oPage->aPages[$iContent]['sLinkNameHome'] : ( isset( $oPage->aPages[$iContent]['sLinkNameRaw'] ) ? $oPage->aPages[$iContent]['sLinkNameRaw'] : $oPage->aPages[$iContent]['sLinkName'] ) ), $sUrlExt );
          $aData['sHidePages'] = null;
        }
        else{
          $aData['sPages'] = '<li></li>';
          $aData['sHidePages'] = ' hide';
        }

        if( $GLOBALS['config']['sorting_products'] === true && isset( $oPage->aPages[$iContent] ) ){
          $aData['sPageLinkName'] = ( isset( $oPage->aPages[$iContent]['sLinkNameHome'] ) ? $oPage->aPages[$iContent]['sLinkNameHome'] : $oPage->aPages[$iContent]['sLinkName'] ).( isset( $GLOBALS['sPhrase'] ) && !empty( $GLOBALS['sPhrase'] ) ? '&amp;sPhrase='.$GLOBALS['sPhrase'].'&amp;' : '&amp;' );
          $oTpl->setIf( 'SORTING' );
        }

        $oTpl->setVariables( 'aData', $aData );
        return $oTpl->tSubBlock( $sFile, $sBlock, 'LIST', 'head' ).$content.$oTpl->tSubBlock( $sFile, $sBlock, 'LIST', 'foot' );
      }
    }
  } // end function listProducts

  /**
  * Return page data
  * @return array
  * @param int  $iProduct
  */
  public function throwProduct( $iProduct ){
    if( isset( $this->aProducts[$iProduct] ) ){
      $aData = $this->aProducts[$iProduct];
      $aData['aCategories'] = $this->aProductsPages[$iProduct];
      $aData['sPrice'] = is_numeric( $this->aProducts[$iProduct]['mPrice'] ) ? displayPrice( $this->aProducts[$iProduct]['mPrice'] ) : $this->aProducts[$iProduct]['mPrice'];
      return $aData;
    }
    else
      return null;
  } // end function throwProduct

  /**
  * Return array with products
  * @return array
  * @param bool $bKeysInverted
  */
  protected function generateProductsListArray( $bKeysInverted = null ){
    if( isset( $this->aProducts ) ){
      foreach( $this->aProductsPages as $iProduct => $aData ){
        foreach( $this->aPages as $iValue ){
          if( isset( $aData[$iValue] ) && !isset( $aProducts[$iProduct] ) ){
            $aReturn[] = $iProduct;
            $aProducts[$iProduct] = true;
          }
        } // end foreach
      } // end foreach

      if( isset( $aReturn ) ){
        if( isset( $bKeysInverted ) )
          return $aProducts;
        else
          return $aReturn;
      }
    }
  } // end function generateProductsListArray

  /**
  * Return array with products
  * @return array
  * @param string $sPhrase
  */
  public function generateProductsSearchListArray( $sPhrase ){
    if( isset( $this->aProducts ) ){
      $aExp = explode( ' ', $sPhrase );
      $iCount = count( $aExp );
      for( $i = 0; $i < $iCount; $i++ ){
        $aExp[$i] = trim( $aExp[$i] );
        if( !empty( $aExp[$i] ) )
          $aWords[] = $aExp[$i];
      } // end for

      $iCount = count( $aWords );

      if( isset( $this->aPages ) ){
        $aProductsPages = $this->generateProductsListArray( true );
      }

      foreach( $this->aProducts as $iProduct => $aProduct ){
        $iFound = 0;
        $bSearch = null;

        if( isset( $this->aPages ) ){
          if( isset( $aProductsPages[$iProduct] ) )
            $bSearch = true;
        }
        else
          $bSearch = true;

        if( isset( $bSearch ) ){
          $sSearchData = implode( ' ', $aProduct );
          for( $i = 0; $i < $iCount; $i++ ){
            //if( stristr( $sSearchData, $aWords[$i] ) )
            if( preg_match( '/'.preg_quote( $aWords[$i] ).'/ui', $sSearchData ) )
              $iFound++;
          } // end for
        }

        if( $iFound == $iCount ){
          $aProducts[] = $iProduct;
        }
      }

      if( isset( $aProducts ) )
        return $aProducts;
    }
  } // end function generateProductsSearchListArray

  /**
  * Return products pages tree
  * @return string
  * @param int  $iProduct
  */
  public function throwProductsPagesTree( $iProduct ){
    global $oPage;
    if( isset( $this->aProductsPages[$iProduct] ) ){
      $content = null;
      $oPage->mData = null;
      foreach( $this->aProductsPages[$iProduct] as $iPage ){
        if( isset( $content ) )
          $content .= '<em>|</em>';
        if( isset( $this->mData[$iPage] ) ){
          $content .= $this->mData[$iPage];
        }
        else{
          $sTree = $oPage->throwPagesTree( $iPage );
          if( empty( $sTree ) && isset( $oPage->aPages[$iPage] ) )
            $sTree .= '<a href="'.$oPage->aPages[$iPage]['sLinkName'].'">'.$oPage->aPages[$iPage]['sName'].'</a>';
          $content .= $this->mData[$iPage] = $sTree;
        }
      }

      return $content;
    }
  } // end function throwProductsPagesTree

  /**
  * Sort products
  * @return array
  * @param array  $aProducts
  * @param string $sSort
  */
  public function sortProducts( $aProducts, $sSort = null ){
    $iCount = count( $aProducts );
    $sFunctionSort = 'sort';

    if( $sSort == 'status' ){
      $sKey = 'iStatus';
    }
    elseif( $sSort == 'price' ){
      $sKey = 'mPrice';    
    }
    elseif( $sSort == 'name' ){
      $sKey = 'sName';    
    }
    else{
      $sFunctionSort = 'rsort';
      $sKey = 'iProduct';    
    }

    for( $i = 0; $i < $iCount; $i++ ){
      $mValue = $this->aProducts[$aProducts[$i]][$sKey]; 
      $aSort[$i][0] = $mValue;
      $aSort[$i][1] = $aProducts[$i];
    } // end for

    $sFunctionSort( $aSort );

    for( $i = 0; $i < $iCount; $i++ ){
      $aProducts[$i] = $aSort[$i][1];
    } // end for    
    return $aProducts;
  } // end function sortProducts 
}
?>